@@ -7,7 +7,7 @@ your smart thermostats.
 
 Example:
   import tado.api
-  t = tado.api('Username', 'Password')
+  t = tado.api('Username', 'Password', 'ClientSecret')
   print(t.get_me())
 
 Disclaimer:
@@ -41,6 +41,7 @@ class Tado:
   json_content   = { 'Content-Type': 'application/json'}
   api            = 'https://my.tado.com/api/v2'
   api_acme       = 'https://acme.tado.com/v1'
+  api_minder     = 'https://minder.tado.com/v1'
 
   def __init__(self, username, password, secret):
     self.username = username
@@ -60,6 +61,7 @@ class Tado:
              'scope'         : 'home.user',
              'username'      : self.username }
     request = self.session.post(url, data=data, headers=self.access_headers)
+    request.raise_for_status()
     response = request.json()
     self.access_token = response['access_token']
     self.refresh_token = response['refresh_token']
@@ -70,11 +72,17 @@ class Tado:
   def _api_call(self, cmd, data=False, method='GET'):
     """Perform an API call."""
     def call_delete(url):
-      return self.session.delete(url, headers=self.access_headers)
+      r = self.session.delete(url, headers=self.access_headers)
+      r.raise_for_status()
+      return r
     def call_put(url, data):
-      return self.session.put(url, headers={**self.access_headers, **self.json_content}, data=json.dumps(data))
+      r = self.session.put(url, headers={**self.access_headers, **self.json_content}, data=json.dumps(data))
+      r.raise_for_status()
+      return r
     def call_get(url):
-      return self.session.get(url, headers=self.access_headers)
+      r = self.session.get(url, headers=self.access_headers)
+      r.raise_for_status()
+      return r
 
     url = '%s/%s' % (self.api, cmd)
     if method == 'DELETE':
@@ -87,11 +95,17 @@ class Tado:
   def _api_acme_call(self, cmd, data=False, method='GET'):
     """Perform an API call."""
     def call_delete(url):
-      return self.session.delete(url, headers=self.access_headers)
+      r = self.session.delete(url, headers=self.access_headers)
+      r.raise_for_status()
+      return r
     def call_put(url, data):
-      return self.session.put(url, headers={**self.access_headers, **self.json_content}, data=json.dumps(data))
+      r = self.session.put(url, headers={**self.access_headers, **self.json_content}, data=json.dumps(data))
+      r.raise_for_status()
+      return r
     def call_get(url):
-      return self.session.get(url, headers=self.access_headers)
+      r = self.session.get(url, headers=self.access_headers)
+      r.raise_for_status()
+      return r
 
     url = '%s/%s' % (self.api_acme, cmd)
     if method == 'DELETE':
@@ -101,6 +115,29 @@ class Tado:
     elif method == 'GET':
       return call_get(url).json()
 
+  def _api_minder_call(self, cmd, data=False, method='GET'):
+    """Perform an API call."""
+    def call_delete(url):
+      r = self.session.delete(url, headers=self.access_headers)
+      r.raise_for_status()
+      return r
+    def call_put(url, data):
+      r = self.session.put(url, headers={**self.access_headers, **self.json_content}, data=json.dumps(data))
+      r.raise_for_status()
+      return r
+    def call_get(url):
+      r = self.session.get(url, headers=self.access_headers)
+      r.raise_for_status()
+      return r
+
+    url = '%s/%s' % (self.api_minder, cmd)
+    if method == 'DELETE':
+      return call_delete(url)
+    elif method == 'PUT' and data:
+      return call_put(url, data).json()
+    elif method == 'GET':
+      return call_get(url).json()
+
   def refresh_auth(self):
     """Refresh an active session."""
     url='https://my.tado.com/oauth/token'
@@ -216,6 +253,17 @@ class Tado:
     data = self._api_call('homes/%i/devices' % self.id)
     return data
 
+  def get_device_usage(self):
+    """
+    Get all devices of your home with how they are used
+
+    Returns:
+      list: All devices of home as list of dictionaries
+    """
+
+    data = self._api_call('homes/%i/deviceList' % self.id)
+    return data
+
   def get_early_start(self, zone):
     """
     Get the early start configuration of a zone.
@@ -276,6 +324,32 @@ class Tado:
     data = self._api_call('homes/%i' % self.id)
     return data
 
+  def get_home_state(self):
+    """
+    Get information about the status of the home.
+
+    Returns:
+      dict: A dictionary with the status of the home.
+    """
+    data = self._api_call('homes/%i/state' % self.id)
+    return data
+
+  def set_home_state(self, at_home):
+    """
+    Set at-home/away state
+
+    Args:
+      at_home (bool): True for at HOME, false for AWAY.
+    """
+
+    if at_home:
+      payload = {'homePresence': 'HOME'}
+    else:
+      payload = {'homePresence': 'AWAY'}
+
+    data = self._api_call('homes/%i/presenceLock' % self.id, payload, method='PUT')
+
+
   def get_installations(self):
     """
     It is unclear what this does.
@@ -386,6 +460,20 @@ class Tado:
     data = self._api_call('homes/%i/mobileDevices' % self.id)
     return data
 
+  def get_schedule_timetables(self, zone):
+    """
+    Gets the schedule timetables supported by the zone
+
+    Args:
+      zone (int): The zone ID.
+
+    Returns:
+      dict: The schedule types
+    """
+
+    data = self._api_call('homes/%i/zones/%i/schedule/timetables' % (self.id, zone))
+    return data
+
   def get_schedule(self, zone):
     """
     Get the type of the currently configured schedule of a zone.
@@ -416,6 +504,58 @@ class Tado:
     data = self._api_call('homes/%i/zones/%i/schedule/activeTimetable' % (self.id, zone))
     return data
 
+  def set_schedule(self, zone, schedule):
+    """
+    Set the type of the currently configured schedule of a zone.
+
+    Args:
+      zone (int): The zone ID.
+      schedule (int): The schedule to activate.
+                      The supported zones are currently
+                       0: ONE_DAY
+                       1: THREE_DAY
+                       2: SEVEN_DAY
+                      But the actual mapping should be retrieved via get_schedule_timetables.
+
+    Returns:
+      dict: The new configuration
+    """
+
+    payload = { 'id': schedule }
+    return self._api_call('homes/%i/zones/%i/schedule/activeTimeTable' % (self.id, zone), payload, method='PUT')
+
+  def get_schedule_blocks(self, zone, schedule):
+    """
+    Gets the blocks for the current schedule on a zone
+
+    Args:
+      zone (int):      The zone ID.
+      schedule (int): The schedule ID to fetch
+
+    Returns:
+      list: The blocks for the requested schedule
+    """
+
+    return self._api_call('homes/%i/zones/%i/schedule/timetables/%i/blocks' % (self.id, zone, schedule))
+
+
+  def set_schedule_blocks(self, zone, schedule, blocks):
+    """
+    Sets the blocks for the current schedule on a zone
+
+    Args:
+      zone (int): The zone ID.
+      schedule (int): The schedule ID.
+      blocks (list): The new blocks
+
+    Returns:
+      list: The new configuration
+    """
+
+    payload = blocks
+    return self._api_call('homes/%i/zones/%i/schedule/timetables/%i/blocks' % (self.id, zone, schedule), payload, method='PUT')
+
+
   def get_state(self, zone):
     """
     Get the current state of a zone including its desired and current temperature. Check out the example output for more.
@@ -476,6 +616,42 @@ class Tado:
     data = self._api_call('homes/%i/zones/%i/state' % (self.id, zone))
     return data
 
+  def get_measuring_device(self, zone):
+    """
+    Gets the active measuring device of a zone
+
+    Args:
+      zone (int): The zone ID.
+
+    Returns:
+      dict: A dictionary with the current measuring informations.
+    """
+
+    data = self._api_call('homes/%i/zones/%i/measuringDevice' % (self.id, zone))
+    return data
+
+  def get_default_overlay(self, zone):
+    """
+    Get the default overlay settings of a zone
+
+    Args:
+      zone (int): The zone ID.
+
+    Returns:
+      dict
+
+    Example
+    =======
+    ::
+      {
+         "terminationCondition": {
+           "type": "TADO_MODE"
+         }
+      }
+    """
+    data = self._api_call('homes/%i/zones/%i/defaultOverlay' % (self.id, zone))
+    return data
+
   def get_users(self):
     """Get all users of your home."""
     data = self._api_call('homes/%i/users' % self.id)
@@ -597,6 +773,22 @@ class Tado:
     data = self._api_call('homes/%i/zones' % self.id)
     return data
 
+  def set_zone_name(self, zone, new_name):
+    """
+    Sets the name of the zone
+
+    Args:
+      zone (int): The zone ID.
+      new_name (str): The new name of the zone
+
+    Returns:
+      dict
+    """
+
+    payload = { 'name': new_name }
+    data = self._api_call('homes/%i/zones/%i/details' % (self.id, zone), payload, method='PUT')
+    return data
+
   def set_early_start(self, zone, enabled):
     """
     Enable or disable the early start feature of a zone.
@@ -682,7 +874,35 @@ class Tado:
     """End the manual control of a zone."""
     data = self._api_call('homes/%i/zones/%i/overlay' % (self.id, zone), method='DELETE')
 
-    
+  def get_away_configuration(self, zone):
+    """
+    Get the away configuration for a zone
+
+    Args:
+      zone (int): The zone ID.
+
+    Returns:
+      dict
+    """
+
+    data = self._api_call('homes/%i/zones/%i/awayConfiguration' % (self.id, zone))
+    return data
+
+  def set_open_window_detection(self, zone, enabled, seconds):
+    """
+    Get the open window detection for a zone
+
+    Args:
+      zone (int): The zone ID.
+      enabled (bool): If open window detection is enabled
+      seconds (int): timeout in seconds
+    """
+
+    payload = { 'enabled' : enabled, 'timeoutInSeconds': timeoutInSeconds }
+
+    data = self._api_call('homes/%i/zones/%i/openWindowDetection' % (self.id, zone), data=payload, method='PUT')
+    return data
+
   def get_report(self, zone, date):
     """
     Args:
@@ -696,6 +916,74 @@ class Tado:
     data = self._api_call('homes/%i/zones/%i/dayReport?date=%s' % (self.id, zone, date))
     return data
 
+  def get_heating_circuits(self):
+    """
+    Gets the heating circuits in the current home
+
+    Returns:
+      list of all dictionaries for all heating circuits
+    """
+
+    data = self._api_call('homes/%i/heatingCircuits' % self.id)
+    return data
+
+  def get_incidents(self):
+    """
+    Gets the ongoing incidents in the current home
+
+    Returns:
+      dict: Incident information
+    """
+
+    data = self._api_minder_call('homes/%i/incidents' % self.id)
+    return data
+
+  def get_installations(self):
+    """
+    Gets the ongoing installations in the current home
+
+    Returns:
+      list of all current installations
+    """
+
+    data = self._api_call('homes/%i/installations' % self.id)
+    return data
+
+  def get_temperature_offset(self, device_serial):
+    """
+    Gets the temperature offset of a device
+
+    Returns:
+      dict: A dictionary that returns the offset in 'celsius' and 'fahrenheit'
+
+    Example
+    =======
+    ::
+      {
+           "celsius": 0.0,
+           "fahrenheit": 0.0
+      }
+    """
+
+    data = self._api_call('devices/%s/temperatureOffset' % device_serial)
+    return data
+
+  def set_temperature_offset(self, device_serial, offset):
+    """
+    Sets the temperature offset of a device
+
+    Args:
+      device_serial (Str): The serial number of the device
+      offset (float): the temperature offset to apply in celsius
+
+    Returns:
+      dict: A dictionary that returns the offset in 'celsius' and 'fahrenheit'
+    """
+
+    payload = { 'celsius':  offset }
+
+    return self._api_call('devices/%s/temperatureOffset' % device_serial, payload, method='PUT')
+
   def get_air_comfort(self):
     """
     Get all zones of your home.
